@@ -45,6 +45,7 @@ from pyfakefs.fake_filesystem_unittest import (
     patchfs,
     PatchMode,
 )
+from pyfakefs.helpers import IS_PYPY
 from pyfakefs.tests.fixtures import module_with_attributes
 
 if sys.version_info < (3, 12):
@@ -71,6 +72,25 @@ class TestPatcher(TestCase):
             contents = f.read()
         self.assertEqual("test", contents)
 
+    def test_nested_invocation(self):
+        with Patcher() as patcher:
+            patcher.fs.create_file("/foo/bar", contents="test")
+            self.assertTrue(patcher.fs.exists("/foo/bar"))
+            with Patcher() as patcher1:
+                self.assertTrue(patcher1.fs.exists("/foo/bar"))
+                patcher1.fs.create_file("/foo/baz", contents="test")
+                self.assertTrue(patcher1.fs.exists("/foo/baz"))
+            self.assertTrue(patcher.fs.exists("/foo/baz"))
+
+    def test_nested_invocation_with_args(self):
+        with Patcher() as patcher:
+            patcher.fs.create_file("/foo/bar", contents="test")
+            with self.assertWarnsRegex(
+                UserWarning, "Nested fake filesystem invocation using custom arguments"
+            ):
+                with Patcher(allow_root_user=False):
+                    pass
+
 
 class TestPatchfsArgumentOrder(TestCase):
     @patchfs
@@ -586,6 +606,9 @@ class PauseResumeTest(fake_filesystem_unittest.TestCase):
         self.resume()
         self.assertFalse(os.path.exists(real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
+        self.pause()
+        real_temp_file.close()
+        self.resume()
 
     def test_pause_resume_fs(self):
         fake_temp_file = tempfile.NamedTemporaryFile()
@@ -607,6 +630,9 @@ class PauseResumeTest(fake_filesystem_unittest.TestCase):
         self.fs.resume()
         self.assertFalse(os.path.exists(real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
+        self.fs.pause()
+        real_temp_file.close()
+        self.fs.resume()
 
     def test_pause_resume_contextmanager(self):
         fake_temp_file = tempfile.NamedTemporaryFile()
@@ -618,6 +644,7 @@ class PauseResumeTest(fake_filesystem_unittest.TestCase):
             real_temp_file = tempfile.NamedTemporaryFile()
             self.assertFalse(self.fs.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(real_temp_file.name))
+            real_temp_file.close()
         self.assertFalse(os.path.exists(real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
@@ -631,6 +658,7 @@ class PauseResumeTest(fake_filesystem_unittest.TestCase):
             real_temp_file = tempfile.NamedTemporaryFile()
             self.assertFalse(self.fs.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(real_temp_file.name))
+            real_temp_file.close()
         self.assertFalse(os.path.exists(real_temp_file.name))
         self.assertTrue(os.path.exists(fake_temp_file.name))
 
@@ -662,7 +690,10 @@ class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
             p.resume()
             self.assertFalse(os.path.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(fake_temp_file.name))
-        real_temp_file.close()
+            fake_temp_file.close()
+            p.pause()
+            real_temp_file.close()
+            p.resume()
 
     def test_pause_resume_contextmanager(self):
         with Patcher() as p:
@@ -675,9 +706,10 @@ class PauseResumePatcherTest(fake_filesystem_unittest.TestCase):
                 real_temp_file = tempfile.NamedTemporaryFile()
                 self.assertFalse(p.fs.exists(real_temp_file.name))
                 self.assertTrue(os.path.exists(real_temp_file.name))
+                real_temp_file.close()
             self.assertFalse(os.path.exists(real_temp_file.name))
             self.assertTrue(os.path.exists(fake_temp_file.name))
-        real_temp_file.close()
+            fake_temp_file.close()
 
 
 class TestPyfakefsTestCase(unittest.TestCase):
@@ -791,6 +823,10 @@ class PathlibTest(TestCase):
 
 
 class TestDeprecationSuppression(fake_filesystem_unittest.TestCase):
+    @unittest.skipIf(
+        IS_PYPY and sys.version_info >= (3, 11),
+        "PyPy 3.11 causes a deprecation warning itself",
+    )
     def test_no_deprecation_warning(self):
         """Ensures that deprecation warnings are suppressed during module
         lookup, see #542.
@@ -817,7 +853,6 @@ def load_configs(configs):
     return retval
 
 
-@unittest.skipIf(sys.version_info < (3, 8), "open_code new in Python 3.8")
 class AutoPatchOpenCodeTestCase(fake_filesystem_unittest.TestCase):
     """Test patching open_code in auto mode, see issue #554."""
 
@@ -950,7 +985,6 @@ class TestAbsolutePathOnWindows(fake_filesystem_unittest.TestCase):
         self.assertTrue(pathlib.Path(".").absolute().is_absolute())
 
 
-@unittest.skipIf(sys.version_info < (3, 8), "Not available before Python 3.8")
 class TestClassSetup(fake_filesystem_unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -995,7 +1029,6 @@ class TestTempPathCreation(fake_filesystem_unittest.TestCase):
         self.check_write_tmp_after_reset(OSType.WINDOWS)
 
 
-@unittest.skipIf(sys.version_info < (3, 8), "Not available before Python 3.8")
 class FakeImportTest(fake_filesystem_unittest.TestCase):
     """Checks that a fake module can be imported in AUTO patch mode."""
 
